// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file Newton.h */

#ifndef AMDIS_NEWTON_H
#define AMDIS_NEWTON_H

#include "MemoryManager.h"
#include "CreatorInterface.h"
#include "NonLinSolver.h"

namespace AMDiS {

  class FiniteElemSpace;

  template<typename VectorType> class OEMSolver;
 
  // ============================================================================
  // ===== class Newton =========================================================
  // ============================================================================

  /**
   * \ingroup Solver
   * 
   * \brief
   * Implements the newton method for solving a non linear system. Sub class of
   * NonLinSolver.
   */
  template<typename VectorType>
  class Newton : public NonLinSolver<VectorType>
  {
  public:
    MEMORY_MANAGED(Newton<VectorType>);

    /** \brief
     * Creator class used in the NonLinSolverMap.
     */
    class Creator : public NonLinSolverCreator<VectorType> 
    {
    public:
      MEMORY_MANAGED(Creator);

      virtual ~Creator() {};

      /** \brief
       * Returns a new Newton object.
       */
      NonLinSolver<VectorType>* create() { 
	return NEW Newton(this->name, this->linearSolver, this->nonLinUpdater); 
      };
    };

    /** \brief
     * Calls constructor of base class NonLinSolver
     */
    Newton(const ::std::string& name_, 
	   OEMSolver<VectorType> *linSolver_,
	   NonLinUpdater<VectorType> *updater)
      : NonLinSolver<VectorType>(name_, linSolver_, updater)
    {};

  private:
    /** \brief
     * realisation of NonLinSolver::init
     */
    void init();

    /** \brief
     * realisation of NonLinSolver::nlsolve
     */
    int  nlsolve(MatVecMultiplier<VectorType> *matVec,
		 VectorType *x, VectorType *rhs,
		 Preconditioner<VectorType> *lPrecon = NULL,
		 Preconditioner<VectorType> *rPrecon = NULL);

    /** \brief
     * realisation of NonLinSolver::exit
     */
    void exit();

  private:
    /** \brief
     * internal used data
     */
    VectorType *b;
  };

}

#include "Newton.hh"

#endif // AMDIS_NEWTON_H
