// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file OResSolver.h */

#ifndef AMDIS_ORESSOLVER_H
#define AMDIS_ORESSOLVER_H

#include "OEMSolver.h"
#include "MemoryManager.h"

namespace AMDiS {

  // ============================================================================
  // ===== class OResSolver =====================================================
  // ============================================================================

  /** 
   * \ingroup Solver
   *
   * \brief
   * Solves a linear system by the method of orthogonal residuals and can be used
   * for symmetric system matrices.
   */
  template<typename VectorType>
  class OResSolver : public OEMSolver<VectorType>
  {
  public:
    MEMORY_MANAGED(OResSolver<VectorType>);

    /** \brief
     * Creator class used in the OEMSolverMap.
     */
    class Creator : public OEMSolverCreator<VectorType>
    {
    public:
      MEMORY_MANAGED(Creator);

      virtual ~Creator() {};

      /** \brief
       * Returns a new OResSolver object.
       */   
      OEMSolver<VectorType>* create() { 
	return NEW OResSolver<VectorType>(this->name);
      };
    };

    /** \brief
     * constructor
     */
    OResSolver(::std::string name);

    /** \brief
     * destructor
     */
    ~OResSolver();

  protected:
    /** \brief
     * realisation of OEMSolver::solveSystem
     */
    int solveSystem(MatVecMultiplier<VectorType> *mv, VectorType *x, VectorType *b);

    /** \brief
     * realisation of OEMSolver::init
     */
    void init();

    /** \brief
     * realisation of OEMSolver::exit
     */
    void exit();

  private:
    // pointer to memory needed for solveSystem
    VectorType *Ad, *d, *d_old, *r, *h, *Ad_old, *Ah;
  };

}

#include "OResSolver.hh"

#endif // AMDIS_ORESSOLVER_H
